/******************************************************************************
* Copyright (C) 2019 - 2022 Xilinx, Inc.  All rights reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/

/*****************************************************************************/
/**
* @file xaie_events.h
* @{
*
* Header file for AIE events module.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who      Date     Changes
* ----- ------   -------- -----------------------------------------------------
* 1.0   Dishita  11/21/2019  Initial creation
* 1.1   Nishad   06/02/2020  Move event files under events directory
* 1.2   Nishad   06/02/2020  Remove unused included header files
* 1.3   Nishad   07/06/2020  Add APIs to configure stream switch port event
*			     selection, event generation, and combo event
*			     registers.
* 1.4   Nishad   07/12/2020  Add APIs to configure event broadcast, PC event,
*			     and group event registers.
* 1.5   Nishad  07/14/2020  Add APIs to reset individual stream switch port
*			    event selection ID and combo event.
* 1.6   Nishad   07/23/2020  Add API to block brodcast signals using bitmap.
* </pre>
*
******************************************************************************/
#ifndef XAIE_EVENTS_H
#define XAIE_EVENTS_H

/***************************** Include Files *********************************/
#include "xaiegbl.h"

/***************************** Macro Definitions *****************************/
#define XAIE_EVENT_INVALID		0xFFFFU

#define XAIE_EVENT_GENERATE_ENUM(ENUM)		XAIE_EVENT_##ENUM,
#define XAIE_EVENT_GENERATE_ENUM_STRING(STRING)	[XAIE_EVENT_##STRING]	=	#STRING,

	/* All core module events of aie tile */
#define XAIE_EVENT_GENERATE_CORE_MOD(TARGET)					\
	XAIE_EVENT_GENERATE_##TARGET(NONE_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(TRUE_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_SYNC_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_VALUE_REACHED_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_1_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_2_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_3_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_4_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_5_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_6_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_7_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_8_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_9_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_10_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_11_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_0_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_1_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_2_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_3_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_4_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_5_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_6_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_7_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_PC_EVENT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PC_0_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_1_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_2_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_3_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_4_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_5_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_6_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_7_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_8_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_9_CORE)					\
	XAIE_EVENT_GENERATE_##TARGET(PC_RANGE_0_1_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PC_RANGE_2_3_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PC_RANGE_4_5_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PC_RANGE_6_7_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PC_RANGE_8_9_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_CORE_STALL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(MEMORY_STALL_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(STREAM_STALL_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(CASCADE_STALL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_STALL_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(DEBUG_HALTED_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(ACTIVE_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(DISABLED_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(ECC_ERROR_STALL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(ECC_SCRUBBING_STALL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_CORE_PROGRAM_FLOW_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_0_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_1_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_2_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_3_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_4_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_5_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_CALL_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_RETURN_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_VECTOR_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_LOAD_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_LOAD_B_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_STORE_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_STREAM_GET_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_STREAM_GET_1_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_STREAM_PUT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_CASCADE_GET_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_CASCADE_PUT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_LOCK_ACQUIRE_REQ_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_LOCK_RELEASE_REQ_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_ERRORS_0_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_ERRORS_1_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_ERRORS_2_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(SRS_SATURATE_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(UPS_SATURATE_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(FP_OVERFLOW_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(FP_UNDERFLOW_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(FP_INVALID_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(FP_DIV_BY_ZERO_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(TLAST_IN_WSS_WORDS_0_2_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(PM_REG_ACCESS_FAILURE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(STREAM_PKT_PARITY_ERROR_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(CONTROL_PKT_ERROR_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_SLAVE_ERROR_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_DECOMPRSN_ERROR_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_ADDRESS_OUT_OF_RANGE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(PM_ECC_ERROR_SCRUB_CORRECTED_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(PM_ECC_ERROR_SCRUB_2BIT_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(PM_ECC_ERROR_1BIT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PM_ECC_ERROR_2BIT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PM_ADDRESS_OUT_OF_RANGE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_ACCESS_TO_UNAVAILABLE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_ACCESS_TO_UNAVAILABLE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_2)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_EVENT_3)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_STREAM_SWITCH_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_0_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_0_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_1_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_1_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_1_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_1_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_2_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_2_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_2_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_2_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_3_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_3_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_3_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_3_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_4_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_4_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_4_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_4_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_5_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_5_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_5_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_5_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_6_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_6_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_6_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_6_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_7_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_7_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_7_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_7_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_BROADCAST_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_1_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_2_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_3_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_4_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_5_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_6_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_7_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_8_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_9_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_10_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_11_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_12_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_13_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_14_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_15_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_USER_EVENT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_1_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_2_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_3_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_4_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_5_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_6_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_7_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_0_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_1_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_2_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_3_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_4_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_5_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_6_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_7_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(FP_HUGE_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INT_FP_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(FP_INF_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_WARNING_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_ERROR_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(DECOMPRESSION_UNDERFLOW_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(STREAM_SWITCH_PORT_PARITY_ERROR_CORE)	\
	XAIE_EVENT_GENERATE_##TARGET(PROCESSOR_BUS_ERROR_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_MATRIX_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_MOVE_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(INSTR_ALU_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(FATAL_ERROR_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(SPARSITY_OVERFLOW_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DATA_ERROR_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_SCRUB_CORRECTED_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_SCRUB_2BIT_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_1BIT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_2BIT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DM_PARITY_ERROR_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_ERROR_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_ERROR_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TASK_TOKEN_STALL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_WATCHPOINT_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_0_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_1_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_2_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_DMA_ACTIVITY_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_START_BD_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_START_BD_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_START_BD_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_START_TASK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_START_TASK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_START_TASK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_FINISHED_BD_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_FINISHED_BD_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_FINISHED_BD_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_FINISHED_TASK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_FINISHED_TASK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_FINISHED_TASK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STALLED_LOCK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STALLED_LOCK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STALLED_LOCK_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STREAM_STARVATION_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STREAM_STARVATION_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STREAM_BACKPRESSURE_CORE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_MEMORY_BACKPRESSURE_CORE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_MEMORY_BACKPRESSURE_CORE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_MEMORY_STARVATION_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_RUNNING_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_RUNNING_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_RUNNING_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_LOCK_CORE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_ACQ_EQ_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_ACQ_GE_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_REL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_EQUAL_TO_VALUE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_ACQ_EQ_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_ACQ_GE_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_REL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_EQUAL_TO_VALUE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_ACQ_EQ_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_ACQ_GE_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_REL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_EQUAL_TO_VALUE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_ACQ_EQ_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_ACQ_GE_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_REL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_EQUAL_TO_VALUE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_ACQ_EQ_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_ACQ_GE_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_REL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_EQUAL_TO_VALUE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_ACQ_EQ_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_ACQ_GE_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_REL_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_EQUAL_TO_VALUE_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_MEMORY_CONFLICT_CORE)		\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_0_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_1_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_2_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_3_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_4_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_5_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_6_CORE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_7_CORE)

	/* All memory module events of aie tile */
#define XAIE_EVENT_GENERATE_MEM_MOD(TARGET)					\
	XAIE_EVENT_GENERATE_##TARGET(NONE_MEM)					\
	XAIE_EVENT_GENERATE_##TARGET(TRUE_MEM)					\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_0_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_SYNC_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_VALUE_REACHED_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_0_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_1_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_0_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_1_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_2_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_3_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_WATCHPOINT_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_0_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_1_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_DMA_ACTIVITY_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_START_BD_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_START_BD_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_START_BD_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_START_BD_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_FINISHED_BD_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_FINISHED_BD_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_FINISHED_BD_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_FINISHED_BD_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_GO_TO_IDLE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_GO_TO_IDLE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_GO_TO_IDLE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_GO_TO_IDLE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STALLED_LOCK_ACQUIRE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STALLED_LOCK_ACQUIRE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STALLED_LOCK_ACQUIRE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_STALLED_LOCK_ACQUIRE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_MEMORY_CONFLICT_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_MEMORY_CONFLICT_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_MEMORY_CONFLICT_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_MEMORY_CONFLICT_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_LOCK_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_0_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_0_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_1_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_1_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_2_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_2_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_3_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_3_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_4_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_4_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_5_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_5_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_6_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_6_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_7_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_7_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_8_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_8_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_9_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_9_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_10_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_10_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_11_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_11_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_12_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_12_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_13_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_13_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_14_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_14_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_15_ACQ_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_15_REL_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_MEMORY_CONFLICT_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_0_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_1_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_2_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_3_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_4_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_5_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_6_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_7_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_ERRORS_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_SCRUB_CORRECTED_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_SCRUB_2BIT_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_1BIT_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_2BIT_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DM_PARITY_ERROR_BANK_2_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_PARITY_ERROR_BANK_3_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_PARITY_ERROR_BANK_4_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_PARITY_ERROR_BANK_5_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_PARITY_ERROR_BANK_6_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_PARITY_ERROR_BANK_7_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_ERROR_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_ERROR_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_ERROR_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_ERROR_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_BROADCAST_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_0_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_1_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_2_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_3_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_4_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_5_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_6_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_7_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_8_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_9_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_10_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_11_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_12_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_13_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_14_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_15_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_USER_EVENT_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_0_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_1_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_2_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_3_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_0_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_1_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_START_TASK_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_START_TASK_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_START_TASK_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_START_TASK_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_FINISHED_TASK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_FINISHED_TASK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_FINISHED_TASK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_FINISHED_TASK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STALLED_LOCK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STALLED_LOCK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STALLED_LOCK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_STALLED_LOCK_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STREAM_STARVATION_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STREAM_STARVATION_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STREAM_BACKPRESSURE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_STREAM_BACKPRESSURE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_MEMORY_BACKPRESSURE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_MEMORY_BACKPRESSURE_MEM)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_MEMORY_STARVATION_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_MEMORY_STARVATION_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL6_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL6_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL6_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL7_ACQ_EQ_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL7_ACQ_GE_MEM)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL7_EQUAL_TO_VALUE_MEM)		\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_ERROR_MEM)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TASK_TOKEN_STALL_MEM)			\

	/* All PL/Noc module events */
#define XAIE_EVENT_GENERATE_PL_MOD(TARGET)	\
	XAIE_EVENT_GENERATE_##TARGET(NONE_PL)					\
	XAIE_EVENT_GENERATE_##TARGET(TRUE_PL)					\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_SYNC_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_VALUE_REACHED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_8_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_9_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_10_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT_11_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_DMA_ACTIVITY_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_START_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_START_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_START_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_START_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_START_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_START_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_FINISHED_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_FINISHED_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_FINISHED_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_FINISHED_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_FINISHED_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_FINISHED_BD_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_GO_TO_IDLE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_GO_TO_IDLE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_GO_TO_IDLE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_GO_TO_IDLE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STALLED_LOCK_ACQUIRE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STALLED_LOCK_ACQUIRE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STALLED_LOCK_ACQUIRE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_STALLED_LOCK_ACQUIRE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_LOCK_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_0_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_0_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_1_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_1_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_2_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_2_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_3_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_3_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_4_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_4_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_5_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_5_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_6_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_6_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_7_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_7_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_8_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_8_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_9_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_9_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_10_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_10_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_11_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_11_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_12_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_12_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_13_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_13_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_14_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_14_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_15_ACQUIRED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_15_RELEASED_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_ERRORS_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_SLAVE_TILE_ERROR_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(CONTROL_PKT_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_DECODE_NSU_ERROR_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_SLAVE_NSU_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_UNSUPPORTED_TRAFFIC_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_UNSECURE_ACCESS_IN_SECURE_MODE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_BYTE_STROBE_ERROR_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_STREAM_SWITCH_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_BROADCAST_A_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_8_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_9_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_10_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_11_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_12_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_13_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_14_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_A_15_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_USER_EVENT_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_0_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_1_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_2_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_3_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_4_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_5_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_6_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_7_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_0_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_1_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_2_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_3_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_4_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_5_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_6_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_7_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_START_TASK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_START_TASK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_START_TASK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_START_TASK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_START_TASK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_START_TASK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_STREAM_STARVATION_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_STREAM_STARVATION_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_STREAM_BACKPRESSURE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_STREAM_BACKPRESSURE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_STREAM_BACKPRESSURE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_STREAM_BACKPRESSURE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_MEMORY_BACKPRESSURE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_MEMORY_BACKPRESSURE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_MEMORY_STARVATION_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_MEMORY_STARVATION_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_MEMORY_STARVATION_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_MEMORY_STARVATION_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_0_RUNNING_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_1_RUNNING_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_0_RUNNING_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_1_RUNNING_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_2_RUNNING_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_3_RUNNING_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_0_ACQ_EQ_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_0_ACQ_GE_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_0_EQUAL_TO_VALUE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_1_ACQ_EQ_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_1_ACQ_GE_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_1_EQUAL_TO_VALUE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_2_ACQ_EQ_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_2_ACQ_GE_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_2_EQUAL_TO_VALUE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_3_ACQ_EQ_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_3_ACQ_GE_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_3_EQUAL_TO_VALUE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_4_ACQ_EQ_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_4_ACQ_GE_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_4_EQUAL_TO_VALUE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_5_ACQ_EQ_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_5_ACQ_GE_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_5_EQUAL_TO_VALUE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(STREAM_SWITCH_PARITY_ERROR_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_ERROR_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_ERROR_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_ERROR_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TASK_TOKEN_STALL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_SUBORDINATE_TILE_ERROR_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NSU_ERROR_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_ERROR_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_HW_ERROR_PL)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TRACE_S2MM_START_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TRACE_S2MM_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TRACE_S2MM_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TRACE_S2MM_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TRACE_S2MM_STREAM_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TRACE_S2MM_MEMORY_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TRACE_S2MM_RUNNING_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_GROUP_DMA_ACTIVITY_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_0_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_1_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_0_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_1_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_0_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_1_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_0_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_1_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_0_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_1_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_0_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_1_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_0_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_1_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_0_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_1_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_0_STREAM_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_1_STREAM_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_0_STREAM_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_1_STREAM_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_0_MEMORY_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_1_MEMORY_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_0_MEMORY_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_1_MEMORY_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_GROUP_DMA_ACTIVITY_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_0_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_1_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_0_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_1_START_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_0_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_1_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_0_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_1_FINISHED_BD_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_0_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_1_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_0_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_1_FINISHED_TASK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_0_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_1_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_0_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_1_STALLED_LOCK_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_0_STREAM_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_1_STREAM_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_0_STREAM_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_1_STREAM_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_0_MEMORY_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_1_MEMORY_BACKPRESSURE_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_0_MEMORY_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_1_MEMORY_STARVATION_PL)	\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_GROUP_LOCK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_0_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_0_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_0_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_0_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_1_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_1_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_1_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_1_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_2_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_2_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_2_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_2_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_3_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_3_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_3_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_3_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_4_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_4_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_4_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_4_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_5_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_5_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_5_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_5_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_GROUP_LOCK_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_0_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_0_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_0_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_0_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_1_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_1_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_1_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_1_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_2_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_2_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_2_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_2_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_3_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_3_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_3_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_3_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_4_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_4_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_4_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_4_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_5_ACQ_EQ_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_5_ACQ_GE_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_5_REL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_5_EQUAL_TO_VALUE_PL)		\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_S2MM_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_S2MM_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_MM2S_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_MM2S_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_LOCK_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_LOCK_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_TASK_TOKEN_STALL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_TASK_TOKEN_STALL_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC0_DMA_HW_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(NOC1_DMA_HW_ERROR_PL)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_GROUP_ACTIVITY_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_START_TASK_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_START_TASK_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_FINISHED_BD_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_FINISHED_BD_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_FINISHED_TASK_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_FINISHED_TASK_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_LOCAL_MEMORY_STARVATION_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_REMOTE_MEMORY_BACKPRESSURE_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_LOCAL_MEMORY_BACKPRESSURE_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_REMOTE_MEMORY_STARVATION_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_ERRORS_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_CORE_MASTER_DECODE_ERROR_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_DMA_MASTER_DECODE_ERROR_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_CORE_MASTER_SLAVE_ERROR_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_DMA_MASTER_SLAVE_ERROR_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_ERROR_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_ERROR_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(PM_ECC_ERROR_1BIT_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(PM_ECC_ERROR_2BIT_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(PRIVATE_DM_ECC_ERROR_1BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(PRIVATE_DM_ECC_ERROR_2BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(SHARED_DM_ECC_ERROR_1BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(SHARED_DM_ECC_ERROR_2BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STRAMS_GROUP_ERRORS_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_MASTER_IDLE_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_MASTER_RUNNING_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_MASTER_STALLED_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_MASTER_TLAST_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_SLAVE_IDLE_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_SLAVE_RUNNING_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_SLAVE_STALLED_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_AXIS_SLAVE_TLAST_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PROGRAM_FLOW_GROUP_ERRORS_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_SLEEP_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_INTERRUPT_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_DEBUG_SYS_RESET_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_DEBUG_WAKEUP_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_TIMER1_INTERRUPT_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_TIMER2_INTERRUPT_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_TIMER3_INTERRUPT_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_TIMER4_INTERRUPT_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_REG_WRITE_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_EXCEPTION_TAKEN_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_JUMP_TAKEN_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_JUMP_HIT_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_DATA_READ_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_DATA_WRITE_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PIPLINE_HALTED_DEBUG_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STREAM_GET_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STREAM_PUT_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_A_START_BD_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_B_START_BD_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_A_START_BD_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_B_START_BD_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_A_START_TASK_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_B_START_TASK_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_A_START_TASK_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_B_START_TASK_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_A_FINISHED_BD_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_B_FINISHED_BD_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_A_FINISHED_BD_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_B_FINISHED_BD_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_A_FINISHED_TASK_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_B_FINISHED_TASK_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_A_FINISHED_TASK_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_B_FINISHED_TASK_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_A_LOCAL_MEMORY_STARVATION_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_B_LOCAL_MEMORY_STARVATION_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_A_REMOTE_MEMORY_BACKPRESSURE_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_B_REMOTE_MEMORY_BACKPRESSURE_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_A_LOCAL_MEMORY_BACKPRESSURE_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_B_LOCAL_MEMORY_BACKPRESSURE_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_A_REMOTE_MEMORY_STARVATION_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_B_REMOTE_MEMORY_STARVATION_UC)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_A_RUNNING_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_DM2MM_B_RUNNING_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_A_RUNNING_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2DM_B_RUNNING_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_A_ERROR_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_B_ERROR_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_A_AXI_MM_ERROR_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_B_AXI_MM_ERROR_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_A_ECC_ERROR_1BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_B_ECC_ERROR_1BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_A_ECC_ERROR_2BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(MODULE_B_ECC_ERROR_2BIT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_GROUP_PC_STATUS_EVENT_UC)		\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_0_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_1_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_2_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_3_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_4_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_5_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_RANGE_0_1_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_RANGE_2_3_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_PC_RANGE_4_5_UC)			\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_0_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_1_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_2_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_3_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_4_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_5_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_6_UC)				\
	XAIE_EVENT_GENERATE_##TARGET(CORE_STATUS_7_UC)

	/* All Mem Tile events */
#define XAIE_EVENT_GENERATE_MEM_TILE(TARGET)						\
	XAIE_EVENT_GENERATE_##TARGET(NONE_MEM_TILE)					\
	XAIE_EVENT_GENERATE_##TARGET(TRUE_MEM_TILE)					\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_0_MEM_TILE)					\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_SYNC_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(TIMER_VALUE_REACHED_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT0_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT1_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT2_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT3_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT4_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT5_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT6_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT7_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT8_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT9_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT10_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PERF_CNT11_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_4_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_5_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_6_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(COMBO_EVENT_7_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_0_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_1_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_2_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_3_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_4_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_5_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_6_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(EDGE_DETECTION_EVENT_7_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_WATCHPOINT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(WATCHPOINT_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_DMA_ACTIVITY_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_START_BD_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_START_BD_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_START_BD_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_START_BD_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_START_TASK_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_START_TASK_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_START_TASK_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_START_TASK_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_FINISHED_BD_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_FINISHED_BD_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_FINISHED_BD_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_FINISHED_BD_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_FINISHED_TASK_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_FINISHED_TASK_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_FINISHED_TASK_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_FINISHED_TASK_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_STALLED_LOCK_ACQUIRE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_STALLED_LOCK_ACQUIRE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_STALLED_LOCK_ACQUIRE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_STALLED_LOCK_ACQUIRE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_STREAM_STARVATION_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_STREAM_STARVATION_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_STREAM_BACKPRESSURE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_STREAM_BACKPRESSURE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_MEMORY_BACKPRESSURE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_MEMORY_BACKPRESSURE_MEM_TILE)	\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_MEMORY_STARVATION_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_MEMORY_STARVATION_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL0_RUNNING_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_SEL1_RUNNING_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL0_RUNNING_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_SEL1_RUNNING_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_LOCK_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL0_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL1_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL2_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL3_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL4_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL5_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL6_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL6_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL6_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL6_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL7_ACQ_EQ_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL7_ACQ_GE_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL7_REL_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_SEL7_EQUAL_TO_VALUE_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_STREAM_SWITCH_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_4_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_4_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_4_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_4_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_5_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_5_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_5_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_5_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_6_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_6_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_6_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_6_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_IDLE_7_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_RUNNING_7_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_STALLED_7_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(PORT_TLAST_7_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_MEMORY_CONFLICT_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_0_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_1_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_2_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_3_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_4_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_5_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_6_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_7_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_8_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_9_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_10_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_11_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_12_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_13_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_14_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_15_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_16_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_17_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_18_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_19_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_20_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_21_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_22_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(CONFLICT_DM_BANK_23_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_ERRORS_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_SCRUB_CORRECTED_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_SCRUB_2BIT_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_1BIT_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DM_ECC_ERROR_2BIT_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(DMA_S2MM_ERROR_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_MM2S_ERROR_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(STREAM_SWITCH_PARITY_ERROR_MEM_TILE)		\
	XAIE_EVENT_GENERATE_##TARGET(STREAM_PKT_ERROR_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(CONTROL_PKT_ERROR_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(AXI_MM_SLAVE_ERROR_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(LOCK_ERROR_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(DMA_TASK_TOKEN_STALL_MEM_TILE)			\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_BROADCAST_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_4_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_5_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_6_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_7_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_8_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_9_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_10_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_11_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_12_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_13_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_14_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(BROADCAST_15_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(GROUP_USER_EVENT_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_0_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_1_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_2_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_3_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_4_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_5_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_6_MEM_TILE)				\
	XAIE_EVENT_GENERATE_##TARGET(USER_EVENT_7_MEM_TILE)

#define XAIE_EVENT_DMA_MM2DM_LOCAL_MEMORY_STARVATION_UC XAIE_EVENT_DMA_MM2DM_LOCAL_MEMORY_BACKPRESSURE_UC
#define XAIE_EVENT_DMA_MM2DM_REMOTE_MEMORY_BACKPRESSURE_UC XAIE_EVENT_DMA_MM2DM_REMOTE_MEMORY_STARVATION_UC

/**************************** Type Definitions *******************************/
/*
 * This enum contains all the Events for all modules: Core, Memory of AIE tile
 * MEM tile and PL tile
 */
typedef enum {
	XAIE_EVENT_GENERATE_CORE_MOD(ENUM)
	XAIE_EVENT_GENERATE_MEM_MOD(ENUM)
	XAIE_EVENT_GENERATE_PL_MOD(ENUM)
	XAIE_EVENT_GENERATE_MEM_TILE(ENUM)
	XAIE_EVENT_MAX
} XAie_Events;

/* Enum to capture stream switch port interface */
typedef enum {
	XAIE_STRMSW_SLAVE,
	XAIE_STRMSW_MASTER,
} XAie_StrmPortIntf;

/* Enum to capture different event combo operations */
typedef enum {
	XAIE_EVENT_COMBO_E1_AND_E2,
	XAIE_EVENT_COMBO_E1_AND_NOTE2,
	XAIE_EVENT_COMBO_E1_OR_E2,
	XAIE_EVENT_COMBO_E1_OR_NOTE2
} XAie_EventComboOps;

/* Enum to capture different event combo configs */
typedef enum {
	XAIE_EVENT_COMBO0,
	XAIE_EVENT_COMBO1,
	XAIE_EVENT_COMBO2,
} XAie_EventComboId;

/* Enum to capture event broadcast directions */
typedef enum {
	XAIE_EVENT_BROADCAST_SOUTH = 0b0001U,
	XAIE_EVENT_BROADCAST_WEST  = 0b0010U,
	XAIE_EVENT_BROADCAST_NORTH = 0b0100U,
	XAIE_EVENT_BROADCAST_EAST  = 0b1000U,
	XAIE_EVENT_BROADCAST_ALL   = 0b1111U,
} XAie_BroadcastDir;

/* Enum to capture edge event config */
typedef enum {
	XAIE_EDGE_EVENT_RISING  = 0b0001U,
	XAIE_EDGE_EVENT_FALLING = 0b0010U,
} XAie_EdgeEventTrig;

/************************** Function Prototypes  *****************************/
AieRC XAie_EventGenerate(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events Event);
AieRC XAie_EventComboConfig(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_EventComboId ComboId,
		XAie_EventComboOps Op, XAie_Events Event1, XAie_Events Event2);
AieRC XAie_EventGetComboEventBase(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events *Event);
AieRC XAie_EventComboReset(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_EventComboId ComboId);
AieRC XAie_EventSelectStrmPort(XAie_DevInst *DevInst, XAie_LocType Loc,
		u8 SelectId, XAie_StrmPortIntf PortIntf, StrmSwPortType Port,
		u8 PortNum);
AieRC XAie_EventSelectStrmPortReset(XAie_DevInst *DevInst, XAie_LocType Loc,
		u8 SelectId);
AieRC XAie_EventSelectDmaChannel(XAie_DevInst *DevInst, XAie_LocType Loc,
		u8 SelectId, XAie_DmaDirection DmaDir, u8 ChannelNum);
AieRC XAie_EventSelectDmaChannelReset(XAie_DevInst *DevInst, XAie_LocType Loc,
		u8 SelectId, XAie_DmaDirection DmaDir);
AieRC XAie_EventGetIdlePortEventBase(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events *Event);
AieRC XAie_EventBroadcast(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, u8 BroadcastId, XAie_Events Event);
AieRC XAie_EventBroadcastReset(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, u8 BroadcastId);
AieRC XAie_EventBroadcastBlockDir(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_BroadcastSw Switch, u8 BroadcastId,
		u8 Dir);
AieRC XAie_EventBroadcastBlockMapDir(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_BroadcastSw Switch,
		u32 ChannelBitMap, u8 Dir);
AieRC XAie_EventBroadcastUnblockDir(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_BroadcastSw Switch, u8 BroadcastId,
		u8 Dir);
AieRC XAie_EventGroupReadConfig(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events GroupEvent, u32 *GroupBitMap);
AieRC XAie_EventGroupControl(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events GroupEvent, u32 GroupBitMap);
AieRC XAie_EventGroupReset(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events GroupEvent);
AieRC XAie_EventEdgeControl(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, u8 SelectId, XAie_Events Event,
		u8 Trigger);
AieRC XAie_EventPCEnable(XAie_DevInst *DevInst, XAie_LocType Loc,
		u8 PCEventId, u16 PCAddr);
AieRC XAie_EventPCDisable(XAie_DevInst *DevInst, XAie_LocType Loc,
		u8 PCEventId);
AieRC XAie_EventPCReset(XAie_DevInst *DevInst, XAie_LocType Loc,
		u8 PCEventId);
AieRC XAie_EventPhysicalToLogicalConv(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, u16 HwEvent, XAie_Events *EnumEvent);
AieRC XAie_EventLogicalToPhysicalConv_16(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events Event, u16 *HwEvent);
AieRC XAie_EventLogicalToPhysicalConv(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events Event, u16 *HwEvent);
AieRC XAie_EventPhysicalToLogicalConv_16(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, u16 HwEvent, XAie_Events *EnumEvent);
AieRC XAie_EventReadStatus(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, XAie_Events Events, u8 *Status);
AieRC XAie_EventGetUserEventBase(XAie_DevInst *DevInst, XAie_LocType Loc,
	XAie_ModuleType Module, XAie_Events *Event);
AieRC XAie_EventRegStatus(XAie_DevInst *DevInst, XAie_LocType Loc,
		XAie_ModuleType Module, u8 EventRegNo, u32 *Status);

#endif		/* end of protection macro */
