/******************************************************************************
* Copyright (C) 2019 - 2022 Xilinx, Inc.  All rights reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/


#ifdef __cplusplus
extern "C" {
#endif

#include <xaiengine/xaie_clock.h>
#include <xaiengine/xaie_core.h>
#include <xaiengine/xaie_dma.h>
#include <xaiengine/xaie_elfloader.h>
#include <xaiengine/xaie_events.h>
#include <xaiengine/xaie_interrupt.h>
#include <xaiengine/xaie_lite.h>
#include <xaiengine/xaie_locks.h>
#include <xaiengine/xaie_mem.h>
#include <xaiengine/xaie_perfcnt.h>
#include <xaiengine/xaie_plif.h>
#include <xaiengine/xaie_reset.h>
#include <xaiengine/xaie_ss.h>
#include <xaiengine/xaie_timer.h>
#include <xaiengine/xaie_trace.h>
#include <xaiengine/xaie_util_events.h>
#include <xaiengine/xaiegbl.h>
#include <xaiengine/xaiegbl_defs.h>
#include <xaiengine/xaie_lite_util.h>
#include <xaiengine/xaie_txn.h>
#include <xaiengine/xaie_routing.h>

#ifdef __cplusplus
}
#endif
